# Зоопарк v1

### К чему эта задача

Очень базовая задача на работу с классами. Пишем класс, который умеет взаимодействовать с другими классами. Причём другие классы могут быть очень разные - иерархия наследования у них может оказаться очень развесистой. Но наш класс общается с ними через единый интерфейс, благодаря чему отдельной обработки всех случаев писать не требуется.


### Постановка задачи

У вас есть зоопарк разных классов. У них есть общий интерфейс. Вот такой:
```cpp
class Animal {
public:
    virtual string getType() = 0;
    virtual bool isDangerous() = 0;
    virtual ~Animal() {}
};
```

Сколько и каких конкретных зверей будет потом создано - вы не знаете. Но интерфейс к ним всем будет таким.

Нужно написать класс смотрителя зоопарка. Классу передают на вход зверей, он их всех последовательно осматривает и пересчитывает, сколько из них было опасных. Прототип класса вот такой:

```cpp
class ZooKeeper {
public:
    // Создаём смотрителя зоопарка
    ZooKeeper() {}

    // Смотрителя попросили обработать очередного зверя.
    // Если зверь был опасный, смотритель фиксирует у себя этот факт.
    void handleAnimal(Animal* a);

    // Возвращает, сколько опасных зверей было обработано на данный момент.
    int getDangerousCount();
};
```
Публичные методы должны быть ровно такие. Необходимую protected / private часть класса можете создать себе любую.

Пример ожидаемого сценария работы всей конструкции:
```cpp
    ZooKeeper z;
    Monkey *m = new Monkey();
    z.handleAnimal(m);
    delete m;
    m = new Monkey();
    z.handleAnimal(m);
    delete m;
    Lion *l = new Lion();
    z.handleAnimal(l);
    delete l;
    cout << z.getDangerousCount() << endl;
```

Должно напечатать 1, так как зверей было 3, но опасный среди них только лев.


### Файлы к задаче

В этой папке лежит zoo\_task\_01.cpp. Это заготовка, в которую нужно вписать свой класс.
