# Приключенцы v0.2

### К чему эта задача

Ещё одна простая задача про наследование и иерархию классов. Теперь технически задача практически вывернута наизнанку - мы хотим в методах базового класса использовать что-то, что будет доопределено в каждом унаследованном от него сабклассе.


### Легенда

Версия 0.1 чудо-игры была успешно собрана и даже запущена. Пришло время для версии 0.2. В игро-механику решили добавить лекарей *(сражений и ран пока нет, но это неважно)*. После бурных дебатов решили, что делать отдельный класс лекаря не стоит - лечить может любой игрок при выполнении определённых условий.

Поэтому в **базовом** классе появился метод canHeal() - может ли данный конкретный герой кого-либо лечить. Внутрь этого метода в дальнейшем будет спрятана сложная логика *(например, в духе "вообще лечить может, но если устал, то уже не может, а если устал, но прямо очень нужно, то всё-таки может")*.

На сейчас внутри этого метода логика будет предельно проста - лечить могут волшебники (всегда, они же умные) или игроки высокого уровня (если вышло добраться до такого уровня, то уж как-нибудь жизнь должна была научить обращаться с бинтами).


### Постановка задачи

Базовый класс героя-игрока из прошлой задачи был немного улучшен: (а) в нём расставили уместные const-ы, (б) в нём появились методы для реализации обсуждённой выше логики. Методы вот такие:

```cpp
    // Может ли игрок колдовать, зависит от конкретного класса игрока
    virtual bool canCast() const = 0;

    // Проверка, может ли игрок лечить других игроков.
    // Лечить умеет или любой волшебник, или 
    // игрок достаточно высокого уровня.
    bool canHeal() const {
        return canCast() || getLevel() > 10;
    }
```

В методе canHeal() уже написана ровно изложенная выше логика. Метод canCast() предполагается к реализации в сабклассах и к использованию в базовом классе - ровно в этом вся идея, ради этого всё затеяно. Вам нужно всего лишь сделать так, чтобы canCast аккуратно возвращал true для Wizard и false для Knight. После этого всё должно заработать. Итак, задача - отрефакторите и допишите свои классы Knight и Wizard под новые требования.

Движок будет обращаться с вашей реализацией примерно вот так:

```cpp
    Player* players[2];
    players[0] = new Wizard();
    players[0]->setStrength(3);
    players[0]->setLevel(5);
    players[1] = new Knight();
    players[1]->setStrength(6);
    players[1]->setLevel(5);

    for(int i = 0; i < 2; i++) {
        cout << "Can heal: " << players[i]->canHeal() << endl;
    }

    for(int i = 0; i < 2; i++)
        delete players[i];
```

На выходе такого примера ожидается банальное:
```
Can heal: 1
Can heal: 0
```


### Файлы к задаче

В этой папке лежит quest\_task\_02.cpp. Это заготовка, в которую нужно вписать свои классы.
