# Зоопарк v2

### К чему эта задача

В прошлой серии мы научились работать в ситуации "нашему классу подсовывают разные другие классы, с которыми он потом что-нибудь делает". Классы на входе очень разные, но наш класс общается с ними через единый интерфейс, благодаря чему сам остаётся очень простым. Теперь давайте в нём напишем решение типовой задачи "мимо нас проходит поток чего-то разнородного, который нужно классифицировать и пересчитывать".


### Постановка задачи

По-прежнему есть зоопарк разных классов с вот таким общим интерфейсом:
```cpp
class Animal {
public:
    virtual string getType() = 0;
    virtual bool isDangerous() = 0;
    virtual ~Animal() {}
};
```

Сколько и каких конкретных зверей будет потом создано - вы не знаете. Но интерфейс к ним всем будет таким.

Смотритель зоопарка теперь должен стать более внимательным бюрократом и начать вести аккуратный учёт зверей - сколько и каких у него есть. Плюс уметь быстро ответить, сколько и кого у него в зоопарке. Прототип класса смотрителя зоопарка теперь вот такой:

```cpp
class ZooKeeper {
public:
    // Создаём смотрителя зоопарка
    ZooKeeper() {}

    // Смотрителя попросили обработать очередного зверя.
    // Если зверь был опасный, смотритель фиксирует у себя этот факт.
    void handleAnimal(Animal* a);

    // Возвращает, сколько опасных зверей было обработано на данный момент.
    int getDangerousCount();

    // Возвращает, сколько зверей такого типа было обработано.
    // Если таких не было, возвращает 0.
    int getAnimalCount(string type);
};

```
Публичные методы должны быть ровно такие. Необходимую protected / private часть класса можете создать себе любую. *Хинт - задача имеет решение буквально в 10 строк, в лекциях был ну очень похожий пример.*

Пример ожидаемого сценария работы всей конструкции:
```cpp
    ZooKeeper z;
    Monkey *m = new Monkey();
    z.handleAnimal(m);
    delete m;
    m = new Monkey();
    z.handleAnimal(m);
    delete m;
    Lion *l = new Lion();
    z.handleAnimal(l);
    delete l;
    cout << z.getAnimalCount("monkey") << endl;
    cout << z.getAnimalCount("lion") << endl;
    cout << z.getAnimalCount("cat") << endl;
```

Должно напечатать:
```
2
1
0
```

Потому что обезьян было две, лев один, а котиков не было совсем.

### Файлы к задаче

В этой папке лежит zoo\_task\_02.cpp. Это заготовка, в которую нужно вписать свой класс.
