# Активность пользователей

### К чему эта задача

Ещё одна задача для оценки на личном опыте, стоит пользоваться STL, или лучше всё писать самостоятельно с нуля.


### Легенда

В прошлой серии мы успешно решили задачу мониторинга пользовательских сессий (учёт входа и выхода пользователя). Этот успех так всех вдохновил, что теперь поставлена новая задача - собирать статистику активности каждого пользователя в системе. *(Задача, разумеется, упрощена относительно того, что захочется в реальности. Из самого заметного - в данной задаче в учёте активности нет разбиения на сессии, что в реальной жизни было бы очень странно.)*


### Постановка задачи

Напишите класс, мониторящий клики пользователей, вот с таким прототипом:

```cpp
class Tracker
{
public:
    Tracker();

    // При любом действии пользователя вызывается этот метод.
    // Методу передаётся:
    // - какой пользователь совершил действие (username);
    // - когда (timestamp - для простоты условные секунды от начала времён).
    // Внимание: не гарантируется, что timestamp расположены строго по
    // возрастанию, в них может быть любой бардак.
    void click(string username, unsigned long long timestamp);

    // По имени пользователя нужно вернуть, сколько всего было кликов
    unsigned long long getClickCount(string username);

    // Когда был первый клик
    unsigned long long getFirstClick(string username);

    // Когда был последний клик
    unsigned long long getLastClick(string username);
};
```

При каждом клике пользователя наш класс уведомляют об этом, вызвав метод click. Поток кликов никак не упорядочен - ни по пользователям, ни по временам событий. Поток кликов нужно как-нибудь учесть, чтобы потом в любой момент уметь отдать по имени пользователя, сколько всего было кликов у данного логина, когда был первый из них и когда последний.


Пример ожидаемого сценария работы всей конструкции:
```cpp
    Tracker t;
    t.click("alice", 1000);
    t.click("bob", 1100);
    t.click("alice", 1001);
    t.click("alice", 1200);
    t.click("alice", 1002);
    cout << t.getClickCount("alice") << endl;
    cout << t.getClickCount("bob") << endl;
    cout << t.getFirstClick("alice") << endl;
    cout << t.getFirstClick("bob") << endl;
    cout << t.getLastClick("alice") << endl;
    cout << t.getLastClick("bob") << endl;
```

В данном примере должна получиться вот такая статистика:
```
4
1
1000
1100
1200
1100
```

### Файлы к задаче

В этой папке лежит activity\_task.cpp. Это заготовка, в которую нужно вписать свой класс.
